package hSPEA2;

import java.util.ArrayList;

public class Experimenter {
	
	public void run_SPEA2(String instance, double std, int pop_size, int gen, int sim_runs, double T, double sched_conf){
		SPEA2 spea2 = new SPEA2(pop_size,gen,sim_runs,sched_conf,T);
		ArrayList<Individual> pareto_front = spea2.main_loop(instance, std);
		PerformanceManager PM = new PerformanceManager();
		
		//ONVG
		//System.out.println("Overall nondominated vector generation(ONVG)");
		//System.out.println("SPEA2: "+PM.ONVG(pareto_front));
		
		//UD
		//System.out.println("Uniform distribution (UD)");
		//System.out.println("SPEA2: "+PM.UD(pareto_front,0.2));
		
		//NDC
		//System.out.println("Number of distinct choices (NDC)");
		//System.out.println("SPEA2: "+PM.NDC(pareto_front,0.2));
		
		//MS

	}
	
	public void run_NSGA2(String instance, double std, int pop_size, int gen, int sim_runs, double T, double sched_conf){
		NSGA2 nsga2 = new NSGA2(pop_size,gen,sim_runs,sched_conf,T);
		ArrayList<Individual> pareto_front = nsga2.main_loop(instance, std);
		PerformanceManager PM = new PerformanceManager();
		
		//ONVG
		System.out.println("Overall nondominated vector generation(ONVG)");
		System.out.println("NSGA2: "+PM.ONVG(pareto_front));
				
		//UD
		System.out.println("Uniform distribution (UD)");
		System.out.println("NSGA2: "+PM.UD(pareto_front,0.2));
				
		//NDC
		System.out.println("Number of distinct choices (NDC)");
		System.out.println("NSGA2: "+PM.NDC(pareto_front,0.2));
				
		//MS

	}
	
	public void run_HGA(String instance, double std, int pop_size, int gen, int sim_runs, double T, double sched_conf, int m){
		HybridGA HGA = new HybridGA(pop_size,gen,sim_runs,sched_conf,T,m);
		HGA.main_loop(instance, std);
	}
	
	public void compare_SPEA2_NSGA2(String instance, double std, int pop_size, int gen, int sim_runs, double T, double sched_conf){
		NSGA2 nsga2 = new NSGA2(pop_size,gen,sim_runs,sched_conf,T);
		SPEA2 spea2 = new SPEA2(pop_size,gen,sim_runs,sched_conf,T);
		
		ArrayList<Individual> X_nsga2 = nsga2.main_loop(instance, std);
		ArrayList<Individual> X_spea2 = spea2.main_loop(instance, std);
		
		
		PerformanceManager PM = new PerformanceManager();
		PM.evaluate_population(X_spea2, std, 100);
		PM.evaluate_population(X_nsga2, std, 100);
		
		System.out.println(PM.ONVG(X_spea2)+", "+PM.ONVG(X_nsga2)+", "+PM.RNI(X_spea2)+", "+PM.RNI(X_nsga2)+", "+PM.NR(X_spea2,X_nsga2)+", "+PM.NR(X_nsga2,X_spea2)+", "+PM.UD(X_spea2,5)+", "+PM.UD(X_nsga2,5)+", "+PM.NDC(X_spea2,0.2)+", "+PM.NDC(X_nsga2,0.2)+", "+PM.MS(X_spea2)+", "+PM.MS(X_nsga2)+", "+PM.C(X_spea2,X_nsga2)+", "+PM.C(X_nsga2,X_spea2));
		
		
		//System.out.println("SPEA2, final population:");
		//PM.evaluate_population(X_spea2, 0.00001,100);
		//System.out.println("NSGA2, final population:");
		//PM.evaluate_population(X_nsga2, 0.00001,100);
		
		//ONVG
		//System.out.println("Overall nondominated vector generation(ONVG)");
		//System.out.println("SPEA2: "+PM.ONVG(X_spea2));
		//System.out.println("NSGA2: "+PM.ONVG(X_nsga2));
		
		//RNI
		//System.out.println("Ratio of nondominatied indviduals(RNI)");
		//System.out.println("SPEA2: "+PM.RNI(X_spea2));
		//System.out.println("NSGA2: "+PM.RNI(X_nsga2));
		
		//NR
		//System.out.println("Pareto dominance indicator (NR)");
		//System.out.println("SPEA2: "+PM.NR(X_spea2,X_nsga2));
		//System.out.println("NSGA2: "+PM.NR(X_nsga2,X_spea2));
		
		//UD
		//System.out.println("Uniform distribution (UD)");
		//System.out.println("SPEA2: "+PM.UD(X_spea2,5));
		//System.out.println("NSGA2: "+PM.UD(X_nsga2,5));
		
		//NDC
		//System.out.println("Number of distinct choices (NDC)");
		//System.out.println("SPEA2: "+PM.NDC(X_spea2,0.2));
		//System.out.println("NSGA2: "+PM.NDC(X_nsga2,0.2));
		
		//MS
		
		//Hypervolume
		
		//Set coverage
		//System.out.println("Set coverage(C)");
		//System.out.println("SPEA2: "+PM.C(X_spea2,X_nsga2));
		//System.out.println("NSGA2: "+PM.C(X_nsga2,X_spea2));
	}
	
	public void SPEA2_compare_stochastic_deterministic(String instance, double std, int pop_size, int gen, int sim_runs, double T, double sched_conf){
		SPEA2 spea2_deterministic = new SPEA2(pop_size,gen,sim_runs,0.5,T);
		SPEA2 spea2_stochastic_50 = new SPEA2(pop_size,gen,sim_runs,0.5,T);
		SPEA2 spea2_stochastic_65 = new SPEA2(pop_size,gen,sim_runs,0.65,T);
		
		ArrayList<Individual> X_spea2_deterministic = spea2_deterministic.main_loop(instance, 0.0001);
		ArrayList<Individual> X_spea2_stochastic_50 = spea2_stochastic_50.main_loop(instance, std);
		ArrayList<Individual> X_spea2_stochastic_65 = spea2_stochastic_65.main_loop(instance, std);
		
		PerformanceManager PM = new PerformanceManager();
		PM.evaluate_population(X_spea2_deterministic, std, 100);
		PM.evaluate_population(X_spea2_stochastic_50, std, 100);
		PM.evaluate_population(X_spea2_stochastic_65, std, 100);
	}
	
	public void NSGA2_compare_stochastic_deterministic(String instance, double std, int pop_size, int gen, int sim_runs, double T, double sched_conf){
		NSGA2 nsga2_deterministic = new NSGA2(pop_size,gen,sim_runs,0.5,T);
		NSGA2 nsga2_stochastic_50 = new NSGA2(pop_size,gen,sim_runs,0.5,T);
		NSGA2 nsga2_stochastic_65 = new NSGA2(pop_size,gen,sim_runs,0.65,T);
		
		
		ArrayList<Individual> X_nsga2_deterministic = nsga2_deterministic.main_loop(instance, 0.0001);
		ArrayList<Individual> X_nsga2_stochastic_50 = nsga2_stochastic_50.main_loop(instance, std);
		ArrayList<Individual> X_nsga2_stochastic_65 = nsga2_stochastic_65.main_loop(instance, std);
		
		PerformanceManager PM = new PerformanceManager();
		PM.evaluate_population(X_nsga2_deterministic, std, 100);
		PM.evaluate_population(X_nsga2_stochastic_50, std, 100);
		PM.evaluate_population(X_nsga2_stochastic_65, std, 100);
	}
	
	public void NSGA2_explicit_averaging(String instance, double std, int pop_size, int gen, int sim_runs, double T, double sched_conf){
		
		NSGA2 nsga2_1 = new NSGA2(pop_size,gen,1,sched_conf,T);
		NSGA2 nsga2_5 = new NSGA2(pop_size,gen,5,sched_conf,T);
		NSGA2 nsga2_30 = new NSGA2(pop_size,gen,30,sched_conf,T);
		NSGA2 nsga2_100 = new NSGA2(pop_size,gen,100,sched_conf,T);
		
		PerformanceManager PM = new PerformanceManager();
		
		long start =  System.currentTimeMillis();
		
		ArrayList<Individual> X_nsga2_1 = nsga2_1.main_loop(instance, std);
		System.out.println("NSGA, 1 simulation run: "+(System.currentTimeMillis()-start));
		PM.evaluate_population(X_nsga2_1, std, 100);
		start =  System.currentTimeMillis();
		
		ArrayList<Individual> X_nsga2_5 = nsga2_5.main_loop(instance, std);
		System.out.println("NSGA, 5 simulation runs: "+(System.currentTimeMillis()-start));
		PM.evaluate_population(X_nsga2_5, std, 100);
		start =  System.currentTimeMillis();
		
		ArrayList<Individual> X_nsga2_30 = nsga2_30.main_loop(instance, std);
		System.out.println("NSGA, 30 simulation runs: "+(System.currentTimeMillis()-start));
		PM.evaluate_population(X_nsga2_30, std, 100);
		start =  System.currentTimeMillis();
		
		ArrayList<Individual> X_nsga2_100 = nsga2_100.main_loop(instance, std);
		System.out.println("NSGA, 100 simulation runs: "+(System.currentTimeMillis()-start));
		PM.evaluate_population(X_nsga2_100, std, 100);
		start =  System.currentTimeMillis();
	}
	
	public void SPEA2_explicit_averaging(String instance, double std, int pop_size, int gen, int sim_runs, double T, double sched_conf){
		
		SPEA2 spea2_1 = new SPEA2(pop_size,gen,1,sched_conf,T);
		SPEA2 spea2_5 = new SPEA2(pop_size,gen,5,sched_conf,T);
		SPEA2 spea2_30 = new SPEA2(pop_size,gen,30,sched_conf,T);
		SPEA2 spea2_100 = new SPEA2(pop_size,gen,100,sched_conf,T);
		
		PerformanceManager PM = new PerformanceManager();
		
		long start =  System.currentTimeMillis();
		
		ArrayList<Individual> X_spea2_1 = spea2_1.main_loop(instance, std);
		System.out.println("SPEA, 1 simulation run: "+(System.currentTimeMillis()-start));
		PM.evaluate_population(X_spea2_1, std, 100);
		start =  System.currentTimeMillis();
		
		ArrayList<Individual> X_spea2_5 = spea2_5.main_loop(instance, std);
		System.out.println("SPEA, 5 simulation runs: "+(System.currentTimeMillis()-start));
		PM.evaluate_population(X_spea2_5, std, 100);
		start =  System.currentTimeMillis();
		
		ArrayList<Individual> X_spea2_30 = spea2_30.main_loop(instance, std);
		System.out.println("SPEA, 30 simulation runs: "+(System.currentTimeMillis()-start));
		PM.evaluate_population(X_spea2_30, std, 100);
		start =  System.currentTimeMillis();
		
		ArrayList<Individual> X_spea2_100 = spea2_100.main_loop(instance, std);
		System.out.println("SPEA, 100 simulation runs: "+(System.currentTimeMillis()-start));
		PM.evaluate_population(X_spea2_100, std, 100);
		start =  System.currentTimeMillis();
	}
	
	public void NSGA2_implicit_averaging(String instance, double std, int pop_size, int gen, int sim_runs, double T, double sched_conf){
		
		NSGA2 nsga2_50 = new NSGA2(50,gen,sim_runs,sched_conf,T);
		NSGA2 nsga2_200 = new NSGA2(200,gen,sim_runs,sched_conf,T);
		NSGA2 nsga2_500 = new NSGA2(500,gen,sim_runs,sched_conf,T);
		NSGA2 nsga2_1000 = new NSGA2(1000,gen,sim_runs,sched_conf,T);
		
		PerformanceManager PM = new PerformanceManager();
		
		long start =  System.currentTimeMillis();
		
		ArrayList<Individual> X_nsga2_50 = nsga2_50.main_loop(instance, std);
		System.out.println("NSGA, 50 pop: "+(System.currentTimeMillis()-start));
		PM.evaluate_population(X_nsga2_50, std, 100);
		start =  System.currentTimeMillis();
		
		ArrayList<Individual> X_nsga2_200 = nsga2_200.main_loop(instance, std);
		System.out.println("NSGA, 200 pop: "+(System.currentTimeMillis()-start));
		PM.evaluate_population(X_nsga2_200, std, 100);
		start =  System.currentTimeMillis();
		
		ArrayList<Individual> X_nsga2_500 = nsga2_500.main_loop(instance, std);
		System.out.println("NSGA, 500 pop: "+(System.currentTimeMillis()-start));
		PM.evaluate_population(X_nsga2_500, std, 100);
		start =  System.currentTimeMillis();
		
		ArrayList<Individual> X_nsga2_1000 = nsga2_1000.main_loop(instance, std);
		System.out.println("NSGA, 1000 pop: "+(System.currentTimeMillis()-start));
		PM.evaluate_population(X_nsga2_1000, std, 100);
		start =  System.currentTimeMillis();
	}
	
	public void SPEA2_implicit_averaging(String instance, double std, int pop_size, int gen, int sim_runs, double T, double sched_conf){
		
		SPEA2 spea2_50 = new SPEA2(50,gen,sim_runs,sched_conf,T);
		SPEA2 spea2_200 = new SPEA2(200,gen,sim_runs,sched_conf,T);
		SPEA2 spea2_500 = new SPEA2(500,gen,sim_runs,sched_conf,T);
		SPEA2 spea2_1000 = new SPEA2(1000,gen,sim_runs,sched_conf,T);
		
		PerformanceManager PM = new PerformanceManager();
		
		long start =  System.currentTimeMillis();
		
		ArrayList<Individual> X_spea2_50 = spea2_50.main_loop(instance, std);
		System.out.println("SPEA, 50 pop: "+(System.currentTimeMillis()-start));
		PM.evaluate_population(X_spea2_50, std, 100);
		start =  System.currentTimeMillis();
		
		ArrayList<Individual> X_spea2_200 = spea2_200.main_loop(instance, std);
		System.out.println("SPEA, 200 pop: "+(System.currentTimeMillis()-start));
		PM.evaluate_population(X_spea2_200, std, 100);
		start =  System.currentTimeMillis();
		
		ArrayList<Individual> X_spea2_500 = spea2_500.main_loop(instance, std);
		System.out.println("SPEA, 500 pop: "+(System.currentTimeMillis()-start));
		PM.evaluate_population(X_spea2_500, std, 100);
		start =  System.currentTimeMillis();
		
		ArrayList<Individual> X_spea2_1000 = spea2_1000.main_loop(instance, std);
		System.out.println("SPEA, 1000 pop: "+(System.currentTimeMillis()-start));
		PM.evaluate_population(X_spea2_1000, std, 100);
		start =  System.currentTimeMillis();
	}
	
public void NSGA2_external_resource(String instance, double std, int pop_size, int gen, int sim_runs, double T, double sched_conf){
		
		NSGA2 nsga2_0 = new NSGA2(pop_size,gen,sim_runs,sched_conf,0);
		NSGA2 nsga2_2 = new NSGA2(pop_size,gen,sim_runs,sched_conf,2);
		NSGA2 nsga2_5 = new NSGA2(pop_size,gen,sim_runs,sched_conf,5);
		NSGA2 nsga2_100 = new NSGA2(pop_size,gen,sim_runs,sched_conf,100);
		
		PerformanceManager PM = new PerformanceManager();
		
		ArrayList<Individual> X_nsga2_0 = nsga2_0.main_loop(instance, std);
		System.out.println("NSGA, T=0: ");
		PM.evaluate_population(X_nsga2_0, std, 100);
		
		ArrayList<Individual> X_nsga2_2 = nsga2_2.main_loop(instance, std);
		System.out.println("NSGA, T=2: ");
		PM.evaluate_population(X_nsga2_2, std, 100);
		
		ArrayList<Individual> X_nsga2_5 = nsga2_5.main_loop(instance, std);
		System.out.println("NSGA, T=5: ");
		PM.evaluate_population(X_nsga2_5, std, 100);
		
		ArrayList<Individual> X_nsga2_100 = nsga2_100.main_loop(instance, std);
		System.out.println("NSGA, T=100: ");
		PM.evaluate_population(X_nsga2_100, std, 100);
	}
	
public void SPEA2_external_resource(String instance, double std, int pop_size, int gen, int sim_runs, double T, double sched_conf){
	
	SPEA2 spea2_0 = new SPEA2(pop_size,gen,sim_runs,sched_conf,0);
	SPEA2 spea2_2 = new SPEA2(pop_size,gen,sim_runs,sched_conf,2);
	SPEA2 spea2_5 = new SPEA2(pop_size,gen,sim_runs,sched_conf,5);
	SPEA2 spea2_100 = new SPEA2(pop_size,gen,sim_runs,sched_conf,100);
	
	PerformanceManager PM = new PerformanceManager();
	
	ArrayList<Individual> X_spea2_0 = spea2_0.main_loop(instance, std);
	System.out.println("SPEA, T=0: ");
	PM.evaluate_population(X_spea2_0, std, 100);
	
	ArrayList<Individual> X_spea2_2 = spea2_2.main_loop(instance, std);
	System.out.println("SPEA, T=2: ");
	PM.evaluate_population(X_spea2_2, std, 100);
	
	ArrayList<Individual> X_spea2_5 = spea2_5.main_loop(instance, std);
	System.out.println("SPEA, T=5: ");
	PM.evaluate_population(X_spea2_5, std, 100);
	
	ArrayList<Individual> X_spea2_100 = spea2_100.main_loop(instance, std);
	System.out.println("SPEA, T=100: ");
	PM.evaluate_population(X_spea2_100, std, 100);
}
	

	public static void main(String[] args){
		//Experimenter e = new Experimenter();
	}
}
