package hSPEA2;

import java.io.BufferedInputStream;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

public class FileReader {
	
	//File reading
    private FileInputStream fis = null;
    private BufferedInputStream bis = null;
    private DataInputStream dis = null;
    private ArrayList<String[]> raw_file=new ArrayList<String[]>();
    private ArrayList<ArrayList<String>> trimmed_file=new ArrayList<ArrayList<String>>();
    
    //Parameters
    public int N; //Number of surgeries
    public int D; //Number of days
    public int R; //Number of operating rooms
    public int K; //Number of surgeons
    public int S; //Number of specialties
    public int P; //Number of time periods in a day
    public int O; //Latest possible working period
    double std; //User defined duration std
    
    //Data
    public double[] durations_mean;
    public double[] durations_std;
    public int[] assigned_surgeon;
    public int[] specialty;
    public int[] refferal_date;
    public int[] deadline;
    public int[][] MSS;
    
    //Interpretaed data
    List<Duration> durations;
    double[][] MSS_probability;
    public int initial_OR_idletime;
    public double initial_waiting_cost;
    
	/**
	 * @param name
	 */
	@SuppressWarnings("deprecation")
	public FileReader(String name, double stdev){
		std = stdev;
		File file = new File("C:\\Users\\Jacob\\workspace\\DataMaster\\src\\hSPEA2\\"+name);
		try {
    		fis = new FileInputStream(file);
 
    		// Here BufferedInputStream is added for fast reading.
    		bis = new BufferedInputStream(fis);
    		dis = new DataInputStream(bis);
 
    		// dis.available() returns 0 if the file does not have more lines.
    		while (dis.available() != 0) {
    			raw_file.add(dis.readLine().trim().split(" "));
    		}
    		
    		//Remove blanks
    		for(int i=0;i<raw_file.size();i++){
    			ArrayList<String> newLine=new ArrayList<String>();
    			for(int j=0;j<raw_file.get(i).length;j++){
    				if(!raw_file.get(i)[j].equals("")){
    					newLine.add(raw_file.get(i)[j]);
    				}
    			}
    			if(!newLine.isEmpty()){
    				trimmed_file.add(newLine);
    			}
    		}
    	
    		// dispose all the resources after using them.
    		fis.close();
    		bis.close();
    		dis.close();
    		
    		//Store data in appropriate objects for further use
    		fileInterpreter();
 
    	} catch (FileNotFoundException e) {
    		e.printStackTrace();
    	} catch (IOException e) {
    		e.printStackTrace();
    	}
	}
	
	public void fileInterpreter(){
		N=Integer.parseInt(trimmed_file.get(0).get(0));
		D=Integer.parseInt(trimmed_file.get(0).get(1));
		R=Integer.parseInt(trimmed_file.get(0).get(2));
		K=Integer.parseInt(trimmed_file.get(0).get(3));
		S=Integer.parseInt(trimmed_file.get(0).get(4));
		P=Integer.parseInt(trimmed_file.get(0).get(5));
		O=Integer.parseInt(trimmed_file.get(0).get(6));
		
		durations_mean=new double[N];
		//durations_std=new double[N];
		assigned_surgeon=new int[N];
		specialty=new int[N];
		refferal_date = new int[N];
		deadline = new int[N];
		
		for(int i=1;i<N+1;i++){
			durations_mean[i-1]=Double.parseDouble(trimmed_file.get(i).get(0));
			//durations_std[i-1]=Double.parseDouble(trimmed_file.get(i).get(1));
			assigned_surgeon[i-1]=Integer.parseInt(trimmed_file.get(i).get(1));
			specialty[i-1]=Integer.parseInt(trimmed_file.get(i).get(2));
			refferal_date[i-1]=Integer.parseInt(trimmed_file.get(i).get(3));
			deadline[i-1]=Integer.parseInt(trimmed_file.get(i).get(4));
		}
		
		MSS=new int[R][D];
		int index=0;
		
		for(int i =N+1;i<trimmed_file.size();i++){
			for(int j=0;j<D;j++){
				MSS[index][j]=Integer.parseInt(trimmed_file.get(i).get(j));
			}
			index+=1;
		}
		
		durations = new ArrayList<Duration>();
		for(int i =0;i<N;i++){
			durations.add(new Duration(Math.log(durations_mean[i]), std));
		}
		
		MSS_probability = generate_MSS_probabilities();
		initial_OR_idletime = set_initial_OR_idletime();
		initial_waiting_cost = set_initial_waiting_cost();
	}
	
	private double [][] generate_MSS_probabilities(){
		double[][] MSS_probability = new double[R][D];
		int[] specialty_occurance = new int[S];
		double[] specialty_probability_unit = new double[S];
		for(int r=0;r<R;r++){
			for(int d=0;d<D;d++){
				specialty_occurance[MSS[r][d]-1]+=1;
			}
		}
		for(int s=0;s<S;s++){
			specialty_probability_unit[s]=(double)1/(specialty_occurance[s]);
		}
		double[] specialty_cumulative_value = new double[S];
		for(int s=0;s<S;s++){
			for(int r=0;r<R;r++){
				for(int d=0;d<D;d++){
					if(MSS[r][d]==s+1){
						specialty_cumulative_value[s]+=specialty_probability_unit[s];
						MSS_probability[r][d]+=specialty_cumulative_value[s];
					}
				}
			}
		}
		return MSS_probability;
	}
	
	public int set_initial_OR_idletime(){
		return R*D*P;
	}
	
	public double set_initial_waiting_cost(){
		double waiting_cost = 0;
		for(int i =0; i<N; i++){
			waiting_cost += waiting_cost(refferal_date[i], deadline[i],10);
		}
		return waiting_cost;
	}
	
	public double waiting_cost(double r, double g, double d){
		return Math.pow((d-(-r))/(g-(-r)), 2);
	}
	
	public void sample_durations(int sim_runs){
		for(int i=0;i<N;i++){
			durations.get(i).set_sample_durations(sim_runs);
		}
	}
	
	public void set_std(double v){
		for(int i=0;i<N;i++){
			durations.get(i).set_std(v);
		}
	}
}

