package hSPEA2;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;


public class HybridGA {
	
	int population_size;
	int simulation_runs;
	double scheduling_confidence;
	double T;
	int k;
	int max_generations;
	double winner_selection_probability;
	double mutation_rate;
	int objective;
	ArrayList<Individual> population;
	int tournament_size;

	
	FileReader fr;
	LocalSearch ls;
	
	public HybridGA(int pop_size, int gen, int sim_runs, double sched_conf, double t, int m){
		//Set from experimenter
		population_size = pop_size;
		max_generations = gen;
		simulation_runs = sim_runs;
		scheduling_confidence = sched_conf;
		T=t;
		objective = m;
		
		//Set internally
		winner_selection_probability = 0.9;
		mutation_rate = 0.2;
		tournament_size = 5;
		
		//Initialization
		population = new ArrayList<Individual>();
	}
	
	public void main_loop(String instance, double std){
		long start =  System.currentTimeMillis();
		fr = new FileReader(instance, std);
		PerformanceManager PM = new PerformanceManager();
		
		fr.sample_durations(simulation_runs);
		initialization();
		Individual bestIndividual = elitistSelection();
		//double bestFitness =bestIndividual.get_objective_values().get(objective);
		double bestFitness =bestIndividual.get_weighted_fitness();
		for(int generation=0;generation<max_generations;generation++){
			nextGeneration();
			Individual generationBest = elitistSelection();
			bestIndividual.set_fitness(bestIndividual.chromosome);
			double newFitness = generationBest.get_weighted_fitness();
			//double newFitness = generationBest.get_objective_values().get(objective);
			if(newFitness<bestFitness){
				bestFitness = newFitness;
				bestIndividual = generationBest;
			}
		}
	}
	
	public void nextGeneration(){
		ArrayList<Individual> nextGeneration = new ArrayList<Individual>();
		Individual eliteIndividual = elitistSelection();
		
		nextGeneration.add(eliteIndividual);
		for(int i =0;i<population_size;i++){
			nextGeneration.add(crossover(tournamentSelection(),tournamentSelection()));
		}
		mutate_population();
		//population = nextGeneration;
		population = improvePopulation(nextGeneration);
		fr.sample_durations(simulation_runs);
		fitness_assignment();
	}
	
	//INITIALIZATION
	public void initialization(){
		while (population.size() < population_size){
			population.add(new Individual(fr, population_size, 0,simulation_runs, scheduling_confidence, T));
		}
		//Archive is already empty
	}
	
	//LOCAL SEARCH
	public void local_search(int fitness_index){
		ArrayList<Individual> improved_population = new ArrayList<Individual>();
		for(Individual i : population){
			//improved_population.add(ls.sequential_best_VND(i, 100, fitness_index));
			improved_population.add(ls.random_neighbourhood_search(i));
		}
		population = improved_population;
	}
	
	//FITNESS ASSIGNMENT
	public void fitness_assignment(){
		//for(int run=0;run<simulation_runs;run++){
			for(Individual i : population){
				i.update_file_reader(fr);
				i.set_fitness(i.chromosome);
			}
		//}
	}
	
	public Individual elitistSelection(){
		Individual bestIndividual = population.get(0);
		for(int i=1;i<population_size;i++){
			if (population.get(i).get_weighted_fitness()<bestIndividual.get_weighted_fitness()){
			//if (population.get(i).get_objective_values().get(objective)<bestIndividual.get_objective_values().get(objective)){
				bestIndividual = population.get(i); 
			}
		}
		//bestIndividual.set_fitness(bestIndividual.chromosome);
		return bestIndividual;
	}
	
	private Individual tournamentSelection(){
		ArrayList<Individual> tournamentParticipators = new ArrayList<Individual>();
		for(int i=0;i<tournament_size;i++){
			tournamentParticipators.add(population.get((int)(Math.random()*population_size)));
		}
		Individual tournamentWinner = tournamentParticipators.get(0);
		for(int i=1;i<tournament_size;i++){
			Individual nextParticipant = tournamentParticipators.get(i);
			if(nextParticipant.get_weighted_fitness()<tournamentWinner.get_weighted_fitness()){
			//if(nextParticipant.get_objective_values().get(objective)<tournamentWinner.get_objective_values().get(objective)){
				tournamentWinner = nextParticipant;
			}
		}
		if (Math.random()<winner_selection_probability){
			return tournamentWinner;
		}
		else return tournamentParticipators.get((int)Math.random()*tournament_size);
	}
	
	public Individual crossover(Individual p1, Individual p2){
		Individual parent1 = p1.clone();
		Individual parent2 = p2.clone();
		int lower_index = (int)(Math.random()*(fr.N-0.5)-1);
		int upper_index = lower_index+(int)(Math.random()*(fr.N-lower_index-0.5));
		ArrayList<Integer> parent2_priority = new ArrayList<Integer>();
		for(int i =0;i<parent2.get_chromosome().size();i++){
			parent2_priority.add(parent2.get_chromosome().get(i).getPriority());
		}
		Individual child = new Individual(fr, population_size, 0, simulation_runs, scheduling_confidence, T);
		for(int i=lower_index;i<upper_index;i++){
			child.get_chromosome().set(i,parent1.get_chromosome().get(i));
			for(int j=0;j<parent2_priority.size();j++){
				if (parent1.get_chromosome().get(i).getPriority()==parent2_priority.get(j)){
					parent2_priority.remove(j);
				}
			}
		}
		for(int i=0;i<lower_index;i++){
			child.get_chromosome().set(i,parent2.get_chromosome().get(i));
			child.get_chromosome().get(i).setPriority(parent2_priority.get(i));
		}
		for(int i=upper_index;i<fr.N;i++){
			child.get_chromosome().set(i,parent2.get_chromosome().get(i));
			child.get_chromosome().get(i).setPriority(parent2_priority.get(i-(upper_index-lower_index)));
		}
		
		//child.set_fitness(child.get_chromosome());
		return child;
	}
	
	//VARIATION
	public void mutate_population(){
		ArrayList<Individual> mutated_population = new ArrayList<Individual>();
		for (int i =0; i< population.size(); i++){
			if(Math.random() < mutation_rate){
				Individual mutant = mutate(population.get(i));
				mutant.set_fitness(mutant.get_chromosome());
				mutated_population.add(mutant);
			}
			else{
				mutated_population.add(population.get(i));
			}
		}
		population = mutated_population;
	}
	
	public void replace_identical_individuals(){
		for(int l=0;l<population.size();l++){
			Individual i = population.get(l);
			for(int k=0;k<population.size();k++){
				Individual j = population.get(k);
				if(!i.equals(j) && i.equlidean_distance(j)==0){
					population.remove(j);
					population.add(new Individual(fr, population_size, 0, simulation_runs, scheduling_confidence, T));
				}
			}
		}
	}
	
	public ArrayList<Individual> improvePopulation(ArrayList<Individual> P){
		ArrayList<Individual> improvedPopulation = new ArrayList<Individual>();
		LocalSearch LS = new LocalSearch(fr);
		for(int i =0; i<population_size;i++){
			improvedPopulation.add(LS.sequential_best_VND(P.get(i), 3, objective));
		}
		return improvedPopulation;
	}

	
	public Individual mutate(Individual individual){
		double r = Math.random();
		if(r<0.33){
			return switchRoomDayMutation(individual);
		}
		else if(r<0.66){
			return switchPriorityMutation(individual);
		}
		else return newRoomDayMutation(individual);
	}
	
	public Individual switchPriorityMutation(Individual current){
		int i = (int)(Math.random()*current.get_chromosome().size());
		int j = (int)(Math.random()*current.get_chromosome().size());
		if(fr.specialty[i]==fr.specialty[j] && i!=j){
			int priority1 = current.get_chromosome().get(i).getPriority();
			int priority2 = current.get_chromosome().get(j).getPriority();
			if(priority1 != priority2){
				Individual n = current.clone();
				n.get_chromosome().get(i).setPriority(priority2);
				n.get_chromosome().get(j).setPriority(priority1);
				n.set_fitness(n.get_chromosome());
				current = n;
			}
		}
		return current;
	}
	
	public Individual switchRoomDayMutation(Individual current){
		int i = (int)(Math.random()*current.get_chromosome().size());
		int j = (int)(Math.random()*current.get_chromosome().size());
		if(fr.specialty[i]==fr.specialty[j] && i!=j){
			int day1 = current.get_chromosome().get(i).getDay();
			int day2 = current.get_chromosome().get(j).getDay();
			int room1 = current.get_chromosome().get(i).getRoom();
			int room2 = current.get_chromosome().get(j).getRoom();
			if(day1 != day2){
				Individual n = current.clone();
				n.get_chromosome().get(i).setDay(day2);
				n.get_chromosome().get(j).setDay(day1);
				n.get_chromosome().get(i).setRoom(room2);
				n.get_chromosome().get(j).setRoom(room1);
				n.set_fitness(n.get_chromosome());
				current = n;
			}
		}
		return current;
	}
	
	public Individual newRoomDayMutation(Individual current){
		int i = (int)(Math.random()*current.get_chromosome().size());
		int s = fr.specialty[i];
		double p = Math.random();
		for(int r=0;r<fr.R;r++){
			for(int d=0;d<fr.D;d++){
				if(fr.MSS[r][d]==s && p<fr.MSS_probability[r][d]){
					Individual n = current.clone();
					n.get_chromosome().get(i).setDay(d);
					n.get_chromosome().get(i).setRoom(r);
					n.set_fitness(n.chromosome);
					return n;
				}
			}
		}
		return current;
	}
}
