package hSPEA2;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.List;

public class Individual {
	
	//General Parameters
	int population_size;
	int simulation_runs;
	double scheduling_confidence;
	double T;
	
	//General fields
	ArrayList<Gene> chromosome;
	ArrayList<Double> objective_values;
	ArrayList<Double> normalized_objective_values;
	ArrayList<Double> mean;
	ArrayList<Double> median;
	ArrayList<Double> low_whisker;
	ArrayList<Double> high_whisker;
	ArrayList<Surgery>[][] room_schedule;
	ArrayList<Surgery>[][] surgeon_schedule;
	
	FileReader fr;
	SchedulePrinter sp;
	
	//SPEA2 fields
	int strength_value;
	int raw_fitness;
	double density;
	Double fitness;
	double[] distances;
	int archive_size;
	
	//NSGA2 fields
	private int rank;
	private int domination_counter;
	private double crowding_distance;
	private ArrayList<Individual> dominated_solutions;
	
	@SuppressWarnings("unchecked")
	public Individual(FileReader f,int pop_size, int arc_size, int sim_runs, double sched_conf, double t){
		population_size = pop_size;
		archive_size = arc_size;
		simulation_runs = sim_runs;
		scheduling_confidence = sched_conf;
		T=t;
		
		distances = new double[population_size+archive_size];
		objective_values = new ArrayList<Double>();
		normalized_objective_values= new ArrayList<Double>();
		mean = new ArrayList<Double>();
		median = new ArrayList<Double>();
		low_whisker = new ArrayList<Double>();
		high_whisker = new ArrayList<Double>();
		fr = f;
		sp = new SchedulePrinter(f);
		chromosome = new ArrayList<Gene>();
		chromosome = random_chromosome();
		set_fitness(chromosome);
		this.dominated_solutions = new ArrayList<Individual>();
	    this.crowding_distance = 0;
	}
	
	//----------------GENERAL METHODS----------------
	
	public void set_chromosome(ArrayList<Gene> c){
		this.chromosome = c;
	}
	
	public ArrayList<Gene> get_chromosome(){
		return this.chromosome;
	}
	
	public void set_objective_values(double operatingRoomOvertime, double operatingRoomIdletime,double surgeonOvertime, double surgeonIdletime, double waitingCost){
		if(objective_values.size()==0){
			objective_values.add(0, operatingRoomOvertime);
			objective_values.add(1, operatingRoomIdletime);
			objective_values.add(2, surgeonOvertime);
			objective_values.add(3, surgeonIdletime);
			objective_values.add(4, waitingCost);
		}
		else{
			objective_values.set(0, operatingRoomOvertime);
			objective_values.set(1, operatingRoomIdletime);
			objective_values.set(2, surgeonOvertime);
			objective_values.set(3, surgeonIdletime);
			objective_values.set(4, waitingCost);
		}
	}
	
	public boolean equal_to(Individual other, double margin){
		boolean equal=true;
		for(int m=0;m<5;m++){
			if(this.objective_values.get(m)-margin>other.objective_values.get(m) || this.objective_values.get(m)+margin<other.objective_values.get(m)){
				equal=false;
			}
		}
		return equal;
	}
	
	public boolean dominates(Individual other){
		boolean noWorseThan = true;
		boolean strictlyBetterThan = false;
		for(int i=0;i<this.objective_values.size();i++){
			double this_objective = Math.round(this.objective_values.get(i)*10);
			double other_objective = Math.round(other.objective_values.get(i)*10);
			this_objective=this_objective/10;
			other_objective=other_objective/10;
			//if(this.objective_values.get(i)>other.get_objective_values().get(i)+0.0){
			if(this_objective>other_objective){
				noWorseThan = false;
			}
			//Add some slack to the comparisons not to distinguish practically identical values
			//if(this.objective_values.get(i)<other.get_objective_values().get(i)-0.01){
			if(this_objective<other_objective){
				strictlyBetterThan = true;
			}
		}
		if(noWorseThan && strictlyBetterThan){
			return true;
		}
		else return false;
	}
	
	public boolean weakly_dominates(Individual other){
		boolean worse_than = false;
		for(int i=0;i<this.objective_values.size();i++){
			if(this.objective_values.get(i)>other.get_objective_values().get(i)){
				worse_than = true;
			}
		}
		if(worse_than){
			return false;
		}
		else return true;
	}
	
	public boolean strictly_dominates(Individual other){
		boolean strictlyBetterThan = true;
		for(int i=0;i<this.objective_values.size();i++){
			//Add some slack to the comparisons not to distinguish practically identical values
			if(this.objective_values.get(i)>other.get_objective_values().get(i)){
				strictlyBetterThan = false;
			}
		}
		if(strictlyBetterThan){
			return true;
		}
		else return false;
	}

	public double equlidean_distance(Individual other){
		double d0 = Math.pow(this.get_objective_values().get(0) - other.get_objective_values().get(0), 2);
		double d1 = Math.pow(this.get_objective_values().get(1) - other.get_objective_values().get(1), 2);
		double d2 = Math.pow(this.get_objective_values().get(2) - other.get_objective_values().get(2), 2);
		double d3 = Math.pow(this.get_objective_values().get(3) - other.get_objective_values().get(3), 2);
		double d4 = Math.pow(this.get_objective_values().get(4) - other.get_objective_values().get(4), 2);
		return Math.sqrt(d0+d1+d2+d3+d4);
	}
	
	public ArrayList<Double> get_objective_values(){
		return this.objective_values;
	}
	
	public ArrayList<Gene> random_chromosome(){
		List<Integer> priorities = new ArrayList<Integer>();
		for(int number=0;number<fr.N;number++){
			priorities.add(number);
		}
		java.util.Collections.shuffle(priorities);
		for (int surgery = 0; surgery<fr.N; surgery++){
			int s = fr.specialty[surgery];
			chromosome.add(random_gene(s,priorities.get(surgery)));
		}
		return chromosome;
	}
	
	public Gene random_gene(int s, int priority){
		double p = Math.random();
		for(int r=0;r<fr.R;r++){
			for(int d=0;d<fr.D;d++){
				if(fr.MSS[r][d]==s && p<fr.MSS_probability[r][d]){
					int[] roomDay = new int[2];
					roomDay[0]=r;
					roomDay[1]=d;
					return new Gene(roomDay,priority);
				}
			}
		}
		return new Gene(new int[2], 0);
	}
	
	public void set_fitness(ArrayList<Gene> chromosome){ 
		
		double OR_overtime =0;
		double OR_idle_time = 0;
		double surgeon_overtime =0;
		double s_idle_time = 0;
		double waiting_cost = fr.initial_waiting_cost;
		@SuppressWarnings("unchecked")
		ArrayList<Surgery>[][] r_schedule = new ArrayList[fr.D][fr.R];
		@SuppressWarnings("unchecked")
		ArrayList<Surgery>[][] k_schedule = new ArrayList[fr.D][fr.K];
		@SuppressWarnings("unchecked")
		ArrayList<IdleTimeInterval>[][] surgeon_idle_time = new ArrayList[fr.D][fr.K];   
		@SuppressWarnings("unchecked")
		ArrayList<IdleTimeInterval>[][] room_idle_time = new ArrayList[fr.D][fr.R]; 
		for(int d=0; d < fr.D; d++){
			for(int k=0; k < fr.K; k++){
				surgeon_idle_time[d][k] = new ArrayList<IdleTimeInterval>();
				k_schedule[d][k] = new ArrayList<Surgery>();
			}
			for(int r=0; r < fr.R; r++){
				room_idle_time[d][r] = new ArrayList<IdleTimeInterval>();
				r_schedule[d][r] = new ArrayList<Surgery>();
			}
		}
		double[][] room_ending_time = new double[fr.D][fr.R];
		double[][] surgeon_ending_time = new double[fr.D][fr.K];
		outerloop:
		for(int locus=0; locus<chromosome.size();locus++){
			int surgery = chromosome.get(locus).getPriority();
			int room = chromosome.get(surgery).getRoom();
			int day = chromosome.get(surgery).getDay();
			int surgeon = fr.assigned_surgeon[surgery]-1;
			double duration = fr.durations.get(surgery).inverse_cummulative_probability(scheduling_confidence);
			for(IdleTimeInterval room_interval : room_idle_time[day][room]){
				if(room_interval.get_duration() > duration){
					if(Math.max(surgeon_ending_time[day][surgeon], room_interval.start_time)+ duration < room_interval.end_time){
						//Add idle time
						if(surgeon_ending_time[day][surgeon] < room_interval.start_time){
							surgeon_idle_time[day][surgeon].add(new IdleTimeInterval(surgeon_ending_time[day][surgeon],room_interval.start_time));
						}
						else if(surgeon_ending_time[day][surgeon] > room_interval.start_time){
							room_idle_time[day][room].add(new IdleTimeInterval(room_interval.start_time, surgeon_ending_time[day][surgeon]));
						}
						//Add surgery to schedule
						Surgery s = new Surgery(surgery, surgeon, day, room, Math.max(surgeon_ending_time[day][surgeon], room_interval.start_time),Math.max(surgeon_ending_time[day][surgeon], room_interval.start_time)+ duration,fr.durations.get(surgery)); 
						r_schedule[day][room].add(s);
						k_schedule[day][surgeon].add(s);
						waiting_cost -= waiting_cost_reduce(surgery, day);
						//Update ending time
						surgeon_ending_time[day][surgeon] = Math.max(surgeon_ending_time[day][surgeon], room_interval.start_time)+ duration;
						//Update room interval
						room_interval.set_start_time(surgeon_ending_time[day][surgeon]);
						continue outerloop;
					}
					for(IdleTimeInterval surgeon_interval : surgeon_idle_time[day][surgeon]){
						if(Math.max(surgeon_interval.start_time, room_interval.start_time) + duration < Math.min(room_interval.end_time, surgeon_interval.end_time)){
							//Add idle times
							if(surgeon_interval.start_time < room_interval.start_time){
								surgeon_idle_time[day][surgeon].add(new IdleTimeInterval(surgeon_interval.start_time, room_interval.start_time));
							}
							else if(surgeon_interval.start_time > room_interval.start_time){
								room_idle_time[day][room].add(new IdleTimeInterval(room_interval.start_time, surgeon_interval.start_time));
							}
							//Add surgery to schedule
							Surgery s = new Surgery(surgery, surgeon, day, room, Math.max(surgeon_interval.start_time, room_interval.start_time),Math.max(surgeon_interval.start_time, room_interval.start_time)+duration,fr.durations.get(surgery));
							r_schedule[day][room].add(s);
							k_schedule[day][surgeon].add(s);
							waiting_cost -= waiting_cost_reduce(surgery, day);
							//Update room and surgeon interval
							room_interval.set_start_time(Math.max(surgeon_interval.start_time, room_interval.start_time) + duration);
							surgeon_interval.set_start_time(room_interval.start_time);
							continue outerloop;
						}
					}
				}
			}
			for(IdleTimeInterval surgeon_interval : surgeon_idle_time[day][surgeon]){
				if(surgeon_interval.get_duration() > duration){
					if(Math.max(room_ending_time[day][room], surgeon_interval.start_time)+ duration < surgeon_interval.end_time){
						//Update idle times
						if(room_ending_time[day][room] < surgeon_interval.start_time){
							room_idle_time[day][room].add(new IdleTimeInterval(room_ending_time[day][room],surgeon_interval.start_time));
						}
						else if(room_ending_time[day][room] > surgeon_interval.start_time){
							surgeon_idle_time[day][surgeon].add(new IdleTimeInterval(surgeon_interval.start_time, room_ending_time[day][room]));
						}
						//Add surgery to schedule
						Surgery s = new Surgery(surgery, surgeon, day, room,Math.max(room_ending_time[day][room], surgeon_interval.start_time),Math.max(room_ending_time[day][room], surgeon_interval.start_time)+duration, fr.durations.get(surgery));
						r_schedule[day][room].add(s);
						k_schedule[day][surgeon].add(s);
						waiting_cost -= waiting_cost_reduce(surgery, day);
						//Update ending time
						room_ending_time[day][room] = Math.max(room_ending_time[day][room], surgeon_interval.start_time)+ duration;
						//Update surgeon interval
						surgeon_interval.set_start_time(room_ending_time[day][room]);
						continue outerloop;
					}
				}
			}
			if(Math.max(room_ending_time[day][room], surgeon_ending_time[day][surgeon])+duration < 40){
				if(room_ending_time[day][room]<surgeon_ending_time[day][surgeon]){
					room_idle_time[day][room].add(new IdleTimeInterval(room_ending_time[day][room],surgeon_ending_time[day][surgeon]));
				}
				else if (room_ending_time[day][room]>surgeon_ending_time[day][surgeon]){
					surgeon_idle_time[day][surgeon].add(new IdleTimeInterval(surgeon_ending_time[day][surgeon],room_ending_time[day][room]));
				}
				//Add surgery to schedule
				Surgery s = new Surgery(surgery, surgeon, day, room, Math.max(room_ending_time[day][room], surgeon_ending_time[day][surgeon]),Math.max(room_ending_time[day][room], surgeon_ending_time[day][surgeon])+duration, fr.durations.get(surgery));
				r_schedule[day][room].add(s);
				k_schedule[day][surgeon].add(s);
				waiting_cost -= waiting_cost_reduce(surgery, day);
				//Update the last ending times
				room_ending_time[day][room] = Math.max(room_ending_time[day][room], surgeon_ending_time[day][surgeon])+duration;
				surgeon_ending_time[day][surgeon] = room_ending_time[day][room];
			}
		}

		//OR idle time
		for(int d =0; d<fr.D;d++){
			for(int r =0; r<fr.R;r++){
				for(IdleTimeInterval room_interval: room_idle_time[d][r]){
					OR_idle_time += room_interval.get_duration();
				}
			}
		}
		for(int d =0; d<fr.D;d++){
			for(int r =0; r<fr.R;r++){
				OR_idle_time += Math.max(32 - room_ending_time[d][r],0); 
			}
		}
		//OR idle time
		for(int d =0; d<fr.D;d++){
			for(int k =0; k<fr.K;k++){
				for(IdleTimeInterval surgeon_interval: surgeon_idle_time[d][k]){
					if(surgeon_interval.start_time>0){
						s_idle_time += surgeon_interval.get_duration();
					}
				}
			}			
		}
		//OR overtime
		for(int d =0; d<fr.D;d++){
			for(int r =0; r<fr.R;r++){
				OR_overtime += Math.max(room_ending_time[d][r]-32,0); 
			}
		}
		//OR overtime
		for(int d =0; d<fr.D;d++){
			for(int k =0; k<fr.K;k++){
				surgeon_overtime += Math.max(surgeon_ending_time[d][k]-32,0); 
			}
		}
		set_objective_values(OR_overtime, OR_idle_time, surgeon_overtime, s_idle_time, waiting_cost);
		room_schedule = r_schedule;
		surgeon_schedule = k_schedule;
		monte_carlo_simulation();
	}
	
	public double get_weighted_fitness(){
		return objective_values.get(0)+objective_values.get(1)+objective_values.get(2)+objective_values.get(3)+objective_values.get(4);
	}
	
	public int getSurgeonIdletime(int[][][] surgeon_schedule){
		int idletime = 0;
		for(int d=0; d<fr.D; d++){
			for(int k=0;k<fr.K;k++){
				for(int p=0;p<fr.P;p++){
					if(surgeon_schedule[d][k][p]>0){
						int start_period = p;
						int end_period = start_period;
						for(int subP=start_period;subP<fr.O;subP++){
							if((surgeon_schedule[d][k][subP]==0 && surgeon_schedule[d][k][subP-1]>0) || (subP==fr.P-1 && surgeon_schedule[d][k][subP]>0)){
								end_period = subP-1;
							}
						}
						//System.out.println("Surgeon "+k+" starts at period "+startPeriod+", ends at "+endPeriod+" on day "+d);
						for(int subP=start_period;subP<end_period;subP++){
							if(surgeon_schedule[d][k][subP]==0){
								idletime+=1;
							}
						}
						break;
					}
				}
			}
		}
		return idletime;
	}
	
	public double waiting_cost_reduce(int surgery, int d){
		return waiting_cost(fr.refferal_date[surgery],fr.deadline[surgery],10)-waiting_cost(fr.refferal_date[surgery],fr.deadline[surgery],d);
	}
	
	public double waiting_cost(double r, double g, double d){
		return Math.pow((d-(-r))/(g-(-r)), 2);
	}
	
	public Individual clone(){
		Individual clone = new Individual(fr, population_size, archive_size, simulation_runs, scheduling_confidence, T);
		for(int i=0;i<fr.N;i++){
			int[] roomDay = this.get_chromosome().get(i).getRoomDay().clone();
			int priority = this.get_chromosome().get(i).getPriority();
			clone.get_chromosome().set(i, new Gene(roomDay,priority));
		}
		clone.set_fitness(clone.get_chromosome());
		return clone;
	}
	
	public int weighted_fitness(double w0, double w1, double w2, double w3, double w4){
		return (int) (w0*objective_values.get(0)+w1*objective_values.get(1)+w2*objective_values.get(2)+w3*objective_values.get(3)+w4*objective_values.get(4));
	}
	
	public void monte_carlo_simulation(){
		double average_OR_overtime =0;
		double average_OR_idle_time = 0;
		double average_surgeon_overtime =0;
		double average_s_idle_time = 0;
		double average_waiting_cost = 0;
		Simulator sim = new Simulator(fr, room_schedule, surgeon_schedule, T);
		for(int run=0; run< simulation_runs; run++){
			ArrayList<Double> simulated_fitness = sim.estimated_fitness(run);
			average_OR_overtime += simulated_fitness.get(0)/simulation_runs;
			average_OR_idle_time += simulated_fitness.get(1)/simulation_runs;
			average_surgeon_overtime += simulated_fitness.get(2)/simulation_runs;
			average_s_idle_time += simulated_fitness.get(3)/simulation_runs;
			average_waiting_cost += simulated_fitness.get(4)/simulation_runs;
		}
		set_objective_values(average_OR_overtime, average_OR_idle_time, average_surgeon_overtime, average_s_idle_time, average_waiting_cost);
	}
	
	//-----------------SPEA2 METHODS------------------
	public void set_strength_value(int strength){
		strength_value = strength;
	}
	
	public int get_strength_value(){
		return this.strength_value;
	}
	
	public void set_raw_fitness(int raw){
		raw_fitness = raw;
	}
	
	public int get_raw_fitness(){
		return this.raw_fitness;
	}
	
	public void set_density(double d){
		density = d;
	}
	
	public double get_density(){
		return this.density;
	}
	
	public void set_fitness(double f){
		fitness = f;
	}
	
	public Double get_fitness(){
		return this.fitness;
	}
	
	public void set_distance_to_neighbour(double distance, int k){
		distances[k] = distance;
	}
	
	public void sort_distances(){
		Arrays.sort(distances);
	}
	
	//-----------------NSGA2 METHODS------------------
	
	public void set_crowding_distance(double d){
		this.crowding_distance = d;
	}
	
	public Double get_crowding_distance(){
		return this.crowding_distance;
	}
	
	public void set_domination_counter(int n){
		this.domination_counter=n;
	}

	public int get_domination_counter(){
		return this.domination_counter;
	}
	
	public void increment_domination_counter(){
		this.domination_counter+=1;
	}
	
	public void decrement_domination_counter(){
		this.domination_counter-=1;
	}
	
	public void add_dominated_solution(Individual q){
		this.dominated_solutions.add(q);
	}
	
	public void clear_dominated_solutions(){
		this.dominated_solutions.clear();
	}
	
	public ArrayList<Individual> get_dominated_solutions(){
		return this.dominated_solutions;
	}
	
	public void set_rank(int r){
		this.rank=r;
	}
	
	public int get_rank(){
		return this.rank;
	}
	
	//Hybrid GA methods
	
	public void update_file_reader(FileReader f){
		fr=f;
	}
	
	//NOT USED
	public void printFitnessReport(){
		System.out.println("Weighted fitness: "+weighted_fitness(1, 1, 1, 1, 1));
		System.out.println("Strength: "+strength_value);
		System.out.println("Raw fitness: "+raw_fitness);
		System.out.println("Density: "+density);
		System.out.println("Fitness: "+fitness);
		System.out.println("OR overtime: "+objective_values.get(0));
		System.out.println("OR idletime: "+objective_values.get(1));
		System.out.println("Surgeon overtime: "+objective_values.get(2));
		System.out.println("Surgeon idletime: "+objective_values.get(3));
		System.out.println("Patient waiting cost: "+objective_values.get(4));
	}
	
	public void print_objective_values(){
		System.out.println(objective_values.get(0)+", "+objective_values.get(1)+", "+objective_values.get(2)+", "+objective_values.get(3)+", "+objective_values.get(4));
	}
	
	public void simulate(){
		System.out.println("Anticipated objective values:");
		print_objective_values();
		Simulator sim = new Simulator(fr, room_schedule, surgeon_schedule, T);
		for(int i=0; i< 10; i++){
			ArrayList<Double> simulated_fitness = sim.estimated_fitness(simulation_runs);
			System.out.println("("+simulated_fitness.get(0)+", "+simulated_fitness.get(1)+", "+simulated_fitness.get(2)+", "+simulated_fitness.get(3)+", "+simulated_fitness.get(4)+")");
		}
	}
		
	public void print_realized_fitness(){
		double room_idle_time = 0;
		double room_overtime = 0;
		for(int d = 0; d < fr.D; d++){
			for(int r = 0; r < fr.R; r++){
				if(room_schedule[d][r].size()>0){
					double idle_time = fr.P;
					for(Surgery i: room_schedule[d][r]){
						//idle_time-=i.duration.sample_duration;
					}
					idle_time+=Double.max(room_schedule[d][r].get(room_schedule[d][r].size()-1).realized_end_time-fr.P, 0);
					room_idle_time+=idle_time;
					room_overtime += Double.max(room_schedule[d][r].get(room_schedule[d][r].size()-1).realized_end_time-fr.P, 0);			
				}
			}
		}
		double surgeon_idle_time = 0;
		double surgeon_overtime = 0;
		for(int d = 0; d < fr.D; d++){
			for(int k = 0; k < fr.K; k++){
				if(surgeon_schedule[d][k].size()>0){
					double idle_time = fr.P;
					for(Surgery i: surgeon_schedule[d][k]){
						//idle_time-=i.duration.sample_duration;
					}
					idle_time+=Double.max(surgeon_schedule[d][k].get(surgeon_schedule[d][k].size()-1).realized_end_time-fr.P, 0);
					idle_time -= surgeon_schedule[d][k].get(0).realized_start_time;
					idle_time -= Double.max(fr.P - surgeon_schedule[d][k].get(surgeon_schedule[d][k].size()-1).realized_end_time,0);
					surgeon_idle_time+=idle_time;
					surgeon_overtime += Double.max(surgeon_schedule[d][k].get(surgeon_schedule[d][k].size()-1).realized_end_time-fr.P, 0);
				}
			}
		}
		
		System.out.println("OR idle time: "+room_idle_time);
		System.out.println("OR overtime: "+room_overtime);
		System.out.println("Surgeon idle time: "+surgeon_idle_time);
		System.out.println("Surgeon overtime: "+surgeon_overtime);
	}
	
	
	public void print_as(){
		SchedulePrinter sp = new SchedulePrinter(fr);
		sp.print_as_scheduled(room_schedule);
	}
	
	public void print_schedule(String size){
		GanttChart g = new GanttChart();
		if(size == "small"){
			g.print_small_schedule(room_schedule);
		}
		if(size == "medium"){
			g.print_medium_schedule(room_schedule);
		}
		if(size == "large"){
			g.print_large_schedule(room_schedule);
		}
	}
	
	public void print_simulated_schedule(String size){
		//System.out.println("Simulation runs: "+simulation_runs);
		GanttChart g = new GanttChart();
		if(size == "small"){
			g.print_small_simulated_schedule(room_schedule);
		}
		if(size == "medium"){
			//g.print_medium_simulated_schedule(room_schedule);
		}
	}
	
	//STATISTICS
	public void set_mean_objective_values(double m0, double m1, double m2, double m3, double m4){
		if(mean.size()==0){
			mean.add(m0);
			mean.add(m1);
			mean.add(m2);
			mean.add(m3);
			mean.add(m4);
		}
		else{
			mean.set(0, m0);
			mean.set(1, m1);
			mean.set(2, m2);
			mean.set(3, m3);
			mean.set(4, m4);
		}
	}
	
	public void set_median_objective_values(double m0, double m1, double m2, double m3, double m4){
		if(median.size()==0){
			median.add(m0);
			median.add(m1);
			median.add(m2);
			median.add(m3);
			median.add(m4);
		}
		else{
			median.set(0, m0);
			median.set(1, m1);
			median.set(2, m2);
			median.set(3, m3);
			median.set(4, m4);
		}
	}
	
	public void set_low_whisker_objective_values(double m0, double m1, double m2, double m3, double m4){
		if(low_whisker.size()==0){
			low_whisker.add(m0);
			low_whisker.add(m1);
			low_whisker.add(m2);
			low_whisker.add(m3);
			low_whisker.add(m4);
		}
		else{
			low_whisker.set(0, m0);
			low_whisker.set(1, m1);
			low_whisker.set(2, m2);
			low_whisker.set(3, m3);
			low_whisker.set(4, m4);
		}
	}
	
	public void set_high_whisker_objective_values(double m0, double m1, double m2, double m3, double m4){
		if(high_whisker.size()==0){
			high_whisker.add(m0);
			high_whisker.add(m1);
			high_whisker.add(m2);
			high_whisker.add(m3);
			high_whisker.add(m4);
		}
		else{
			high_whisker.set(0, m0);
			high_whisker.set(1, m1);
			high_whisker.set(2, m2);
			high_whisker.set(3, m3);
			high_whisker.set(4, m4);
		}
	}
	
	public static Comparator<Individual> COMPARE_BY_FITNESS = new Comparator<Individual>() {
        public int compare(Individual one, Individual other) {
            return one.get_fitness().compareTo(other.get_fitness());
        }
    };
    
    public static Comparator<Individual> COMPARE_BY_OR_OVERTIME = new Comparator<Individual>() {
        public int compare(Individual one, Individual other) {
            return one.get_objective_values().get(0).compareTo(other.get_objective_values().get(0));
        }
    };

    public static Comparator<Individual> COMPARE_BY_OR_IDLETIME = new Comparator<Individual>() {
        public int compare(Individual one, Individual other) {
            return one.get_objective_values().get(1).compareTo(other.get_objective_values().get(1));
        }
    };
    
    public static Comparator<Individual> COMPARE_BY_SURGEON_IDLETIME = new Comparator<Individual>() {
        public int compare(Individual one, Individual other) {
            return one.get_objective_values().get(2).compareTo(other.get_objective_values().get(2));
        }
    };
    
    public static Comparator<Individual> COMPARE_BY_SURGEON_OVERTIME = new Comparator<Individual>() {
        public int compare(Individual one, Individual other) {
            return one.get_objective_values().get(3).compareTo(other.get_objective_values().get(3));
        }
    };
    
    public static Comparator<Individual> COMPARE_BY_WAITING_COST = new Comparator<Individual>() {
        public int compare(Individual one, Individual other) {
            return one.get_objective_values().get(4).compareTo(other.get_objective_values().get(4));
        }
    };
    
    public static Comparator<Individual> COMPARE_BY_CROWDING_DISTANCE = new Comparator<Individual>() {
        public int compare(Individual one, Individual other) {
            return one.get_crowding_distance().compareTo(other.get_crowding_distance());
        }
    };

}

