package hSPEA2;

import java.util.ArrayList;
import java.util.Collections;

public class NSGA2 {
	
    //Helper
    private FileReader fr;
    //GA Parameters
    int population_size;
	int simulation_runs;
	double scheduling_confidence;
	double T;
	int max_generations;
	
	//NSGA2 Parameters
    final static int tournamentSize = 10;
    final static double tournamentWinnerProbability = 1;
    final static double ancestorRate = 0.1;
    final static double crossoverRate = 0.9;
    final static double mutation_rate = 0.2;
    final static double localSearchIterations = 50;
    final static int M = 5; //Number of objectives
    //Fields
    private ArrayList<Individual> population;

	public NSGA2(int pop_size, int gen, int sim_runs, double sched_conf, double t){
	    population_size=pop_size;
	    max_generations =gen;
		simulation_runs=sim_runs;
		scheduling_confidence=sched_conf;
		T=t;
	}
	
	public ArrayList<Individual> main_loop(String instance, double std){
		fr = new FileReader(instance, std);
		fr.sample_durations(simulation_runs);
		PerformanceManager PM = new PerformanceManager();
		ArrayList<Individual> P = intitialPopulation(); //Parent population
		for(int iteration=0;iteration<max_generations+1;iteration++){
			ArrayList<Individual> Q = nextGeneration(P);
			ArrayList<Individual> R = new ArrayList<Individual>(); //Combined 
			R.addAll(P);
			R.addAll(Q);
			replaceEqualSolutions(R);
			ArrayList<ArrayList<Individual>> F = fastNonDominatedSort(R);

			ArrayList<Individual> S = new ArrayList<Individual>(); //Next population, initially empty
			int i=0;
			while(S.size()+F.get(i).size()<=population_size){
				crowdingDistanceAssignment(F.get(i));
				S.addAll(F.get(i));
				i+=1;
			}
			int remaining = population_size-S.size();
			for(int j=0;j<remaining;j++){
				S.add(F.get(i).get(j));
			}
			P=S;
			fr.sample_durations(simulation_runs);
		}
		return P;
	}
	
	public void crowdingDistanceAssignment(ArrayList<Individual> H){
		int l = H.size()-1;
		for(Individual p:H){
			p.set_crowding_distance(0);
		}
		for(int m=0;m<M;m++){
			sortOnSingleObjective(H,m);
			H.get(0).set_crowding_distance(Double.POSITIVE_INFINITY);
			H.get(l).set_crowding_distance(Double.POSITIVE_INFINITY);
			double maxMinDifference = H.get(0).get_objective_values().get(m)-H.get(l).get_objective_values().get(m);
			for(int i=1;i<l;i++){
				double distance = H.get(i).get_crowding_distance();
				double newDistance = distance+Math.abs((H.get(i+1).get_objective_values().get(m)-H.get(i-1).get_objective_values().get(m))/maxMinDifference);
				H.get(i).set_crowding_distance(newDistance);
			}
		}
	}
	
	public void sortOnSingleObjective(ArrayList<Individual> H, int m){
		if(m==0){
			Collections.sort(H, Individual.COMPARE_BY_OR_OVERTIME);
		}
		else if(m==1){
			Collections.sort(H, Individual.COMPARE_BY_OR_IDLETIME);
		}
		else if(m==2){
			Collections.sort(H, Individual.COMPARE_BY_SURGEON_IDLETIME);
		}
		else if(m==3){
			Collections.sort(H, Individual.COMPARE_BY_SURGEON_IDLETIME);
		}
		else if(m==4){
			Collections.sort(H, Individual.COMPARE_BY_WAITING_COST);
		}
	}
	
	public void crowdedComparisonSort(ArrayList<Individual> H){
		Collections.sort(H, Individual.COMPARE_BY_CROWDING_DISTANCE);
		Collections.reverse(H);
	}
	
	public ArrayList<ArrayList<Individual>> fastNonDominatedSort(ArrayList<Individual> P){
		ArrayList<ArrayList<Individual>> F = new ArrayList<ArrayList<Individual>>();
		ArrayList<Individual> F1 = new ArrayList<Individual>();
		for(Individual p:P){
			p.clear_dominated_solutions();
			p.set_domination_counter(0);
			for(Individual q:P){
				if(p.dominates(q)){
					p.add_dominated_solution(q);
				}
				else if(q.dominates(p)) {
					p.increment_domination_counter();
				}
			}
			if(p.get_domination_counter()==0){
				p.set_rank(1);
				F1.add(p);
			}
		}
		F.add(F1);
		int i=0;
		while(!F.get(i).isEmpty()){
			ArrayList<Individual> Q = new ArrayList<Individual>();
			for(Individual p:F.get(i)){
				for(Individual q:p.get_dominated_solutions()){
					q.decrement_domination_counter();
					if(q.get_domination_counter()==0){
						q.set_rank(i+2);
						Q.add(q);
					}
				}
			}
			F.add(Q);
			i+=1;
		}
		return F;
	}
	
	public ArrayList<Individual> nextGeneration(ArrayList<Individual> P){
		ArrayList<Individual> Q = new ArrayList<Individual>();
		ArrayList<ArrayList<Individual>> F=fastNonDominatedSort(P);
		for(int i=0;i<(int)(population_size);i++){
			Individual parent1 = binaryTournamentSelection(P);
			Individual parent2 = binaryTournamentSelection(P);
			Q.add(crossover(parent1,parent2));
		}
		Q = mutate_population(Q);
		return Q;
	}
	
	public Individual binaryTournamentSelection(ArrayList<Individual> P){
		Individual contestent1 = P.get((int)(Math.random()*population_size));
		Individual contestent2 = P.get((int)(Math.random()*population_size));
		return crowdedComparisonOperator(contestent1,contestent2);
	}
	
	public Individual crowdedComparisonOperator(Individual p, Individual q){
		if(p.get_rank()<q.get_rank()||(p.get_rank()==q.get_rank()&&p.get_crowding_distance()>q.get_crowding_distance())&& Math.random()<tournamentWinnerProbability){
			return p;
		}
		else{
			return q;
		}
	}
	
	public ArrayList<Individual> intitialPopulation(){
		ArrayList<Individual> initialPopulation = new ArrayList<Individual>();
		for(int i=0;i<population_size;i++){
			initialPopulation.add(new Individual(fr, population_size, 0, simulation_runs, scheduling_confidence, T));
		}
		return initialPopulation;
	}
	
	public Individual crossover(Individual p1, Individual p2){
		if(Math.random()>crossoverRate){
			return p1;
		}
		else{
			Individual parent1 = p1.clone();
			Individual parent2 = p2.clone();
			int lowerIndex = (int)(Math.random()*(fr.N-0.5)-1);
			int upperIndex = lowerIndex+(int)(Math.random()*(fr.N-lowerIndex-0.5));
			ArrayList<Integer> parent2Priority = new ArrayList<Integer>();
			for(int i =0;i<parent2.get_chromosome().size();i++){
				parent2Priority.add(parent2.get_chromosome().get(i).getPriority());
			}
			Individual child = new Individual(fr, population_size, 0, simulation_runs, scheduling_confidence, T);
			for(int i=lowerIndex;i<upperIndex;i++){
				child.get_chromosome().set(i,parent1.get_chromosome().get(i));
				for(int j=0;j<parent2Priority.size();j++){
					if (parent1.get_chromosome().get(i).getPriority()==parent2Priority.get(j)){
						parent2Priority.remove(j);
					}
				}
			}
			for(int i=0;i<lowerIndex;i++){
				child.get_chromosome().set(i,parent2.get_chromosome().get(i));
				child.get_chromosome().get(i).setPriority(parent2Priority.get(i));
			}
			for(int i=upperIndex;i<fr.N;i++){
				child.get_chromosome().set(i,parent2.get_chromosome().get(i));
				child.get_chromosome().get(i).setPriority(parent2Priority.get(i-(upperIndex-lowerIndex)));
			}
		
			child.set_fitness(child.chromosome);
			return child;
			}
	}
	
	public void replaceEqualSolutions(ArrayList<Individual> P){
		for(int i=0;i<P.size();i++){
			for(int j=i+1;j<P.size();j++){
				if(P.get(i).equal_to(P.get(j), 0.5)){
					P.remove(j);
					P.add(new Individual(fr, population_size, 0, simulation_runs, scheduling_confidence, T));
				}
			}
		}
	}
	
	public ArrayList<Individual> mutate_population(ArrayList<Individual> Q){
		ArrayList<Individual> mutated_population = new ArrayList<Individual>();
		for (int i =0; i< Q.size(); i++){
			if(Math.random() < mutation_rate){
				Individual mutant = mutate(Q.get(i));
				mutant.set_fitness(mutant.get_chromosome());
				mutated_population.add(mutant);
			}
			else{
				mutated_population.add(Q.get(i));
			}
		}
		return mutated_population;
	}
	
	public Individual mutate(Individual individual){
		double r = Math.random();
		if(r<0.33){
			return switchRoomDayMutation(individual);
		}
		else if(r<0.66){
			return switchPriorityMutation(individual);
		}
		else return newRoomDayMutation(individual);
	}
	
	public Individual switchPriorityMutation(Individual current){
		int i = (int)(Math.random()*current.get_chromosome().size());
		int j = (int)(Math.random()*current.get_chromosome().size());
		if(fr.specialty[i]==fr.specialty[j] && i!=j){
			int priority1 = current.get_chromosome().get(i).getPriority();
			int priority2 = current.get_chromosome().get(j).getPriority();
			if(priority1 != priority2){
				Individual n = current.clone();
				n.get_chromosome().get(i).setPriority(priority2);
				n.get_chromosome().get(j).setPriority(priority1);
				n.set_fitness(n.get_chromosome());
				current = n;
			}
		}
		return current;
	}
	
	public Individual switchRoomDayMutation(Individual current){
		int i = (int)(Math.random()*current.get_chromosome().size());
		int j = (int)(Math.random()*current.get_chromosome().size());
		if(fr.specialty[i]==fr.specialty[j] && i!=j){
			int day1 = current.get_chromosome().get(i).getDay();
			int day2 = current.get_chromosome().get(j).getDay();
			int room1 = current.get_chromosome().get(i).getRoom();
			int room2 = current.get_chromosome().get(j).getRoom();
			if(day1 != day2){
				Individual n = current.clone();
				n.get_chromosome().get(i).setDay(day2);
				n.get_chromosome().get(j).setDay(day1);
				n.get_chromosome().get(i).setRoom(room2);
				n.get_chromosome().get(j).setRoom(room1);
				n.set_fitness(n.get_chromosome());
				current = n;
			}
		}
		return current;
	}
	
	public Individual newRoomDayMutation(Individual current){
		int i = (int)(Math.random()*current.get_chromosome().size());
		int s = fr.specialty[i];
		double p = Math.random();
		for(int r=0;r<fr.R;r++){
			for(int d=0;d<fr.D;d++){
				if(fr.MSS[r][d]==s && p<fr.MSS_probability[r][d]){
					Individual n = current.clone();
					n.get_chromosome().get(i).setDay(d);
					n.get_chromosome().get(i).setRoom(r);
					n.set_fitness(n.chromosome);
					return n;
				}
			}
		}
		return current;
	}

}
