package hSPEA2;

import java.util.ArrayList;
import java.util.Arrays;

public class PerformanceManager {

	public double RNI(ArrayList<Individual> X){
		int NI=0;
		for(Individual i : X){
			boolean dominated = false;
			for(Individual j : X){
				if(j.dominates(i)){
					dominated = true;
				}
			}
			if(!dominated){
				NI+=1;
			}
		}
		double RNI = (double)NI/X.size();
		return RNI;
	}
	
	public int ONVG(ArrayList<Individual> X){
		int ONVG=0;
		for(Individual i : X){
			boolean dominated = false;
			for(Individual j : X){
				if(j.dominates(i)){
					dominated = true;
				}
			}
			if(!dominated){
				ONVG+=1;
			}
		}
		return ONVG; 
	}
	
	public double NR(ArrayList<Individual> A1, ArrayList<Individual> A2){
		ArrayList<Individual> B = new ArrayList<Individual>();
		int A1_size=0;
		for(Individual b:A1){
			boolean dominated = false;
			for(Individual a1:A1){
				if(a1.dominates(b)){
					dominated = true;
				}
			}
			for(Individual a2:A2){
				if(a2.dominates(b)){
					dominated = true;
				}
			}
			if(!dominated){
				A1_size+=1;
				B.add(b);
			}
		}
		for(Individual b:A2){
			boolean dominated = false;
			for(Individual a1:A1){
				if(a1.dominates(b)){
					dominated = true;
				}
			}
			for(Individual a2:A2){
				if(a2.dominates(b)){
					dominated = true;
				}
			}
			if(!dominated){
				B.add(b);
			}
		}
		double NR=(double)A1_size/B.size();
		return NR;
	}
	
	public double UD(ArrayList<Individual> X, double share){
		return(1/(1+niche_count_std(X,share)));
	}
	
	//Assumes M=5
	public int NDC(ArrayList<Individual> X, double mu){
		ArrayList<Individual> Y = new ArrayList<Individual>(); 
		for(Individual i : X){
			boolean dominated = false;
			for(Individual j : X){
				if(j.dominates(i)){
					dominated = true;
				}
			}
			if(!dominated){
				Y.add(i);
			}
		}
		int NDC=0;
		set_normalized_objective_values(Y);
		for(int m_0=0;m_0<(int)1/mu-1+1;m_0++){
			for(int m_1=0;m_1<(int)1/mu-1+1;m_1++){
				for(int m_2=0;m_2<(int)1/mu-1+1;m_2++){
					for(int m_3=0;m_3<(int)1/mu-1+1;m_3++){
						for(int m_4=0;m_4<(int)1/mu-1+1;m_4++){
							double[] q={m_0/(1/mu),m_1/(1/mu),m_2/(1/mu),m_3/(1/mu),m_4/(1/mu)};
							NDC+=NT(q,mu,Y);
						}
					}
				}
			}
		}
		return NDC;
	}
	
	//Assumes M=5
	public double MS(ArrayList<Individual> X){
		double[] true_min={0,0,0,0,47.46};
		double[] true_max={100,100,100,75,90};
		double min_m0=Double.POSITIVE_INFINITY;
		double min_m1=Double.POSITIVE_INFINITY;
		double min_m2=Double.POSITIVE_INFINITY;
		double min_m3=Double.POSITIVE_INFINITY;
		double min_m4=Double.POSITIVE_INFINITY;
		double max_m0=0;
		double max_m1=0;
		double max_m2=0;
		double max_m3=0;
		double max_m4=0;
		for(Individual x:X){
			if(x.objective_values.get(0)<min_m0){
				min_m0=x.objective_values.get(0);
			}
			if(x.objective_values.get(0)>max_m0){
				max_m0=x.objective_values.get(0);
			}
			if(x.objective_values.get(1)<min_m1){
				min_m1=x.objective_values.get(1);
			}
			if(x.objective_values.get(1)>max_m1){
				max_m1=x.objective_values.get(1);
			}
			if(x.objective_values.get(2)<min_m2){
				min_m2=x.objective_values.get(2);
			}
			if(x.objective_values.get(2)>max_m2){
				max_m2=x.objective_values.get(2);
			}
			if(x.objective_values.get(3)<min_m3){
				min_m3=x.objective_values.get(3);
			}
			if(x.objective_values.get(3)>max_m3){
				max_m3=x.objective_values.get(3);
			}
			if(x.objective_values.get(4)<min_m4){
				min_m4=x.objective_values.get(4);
			}
			if(x.objective_values.get(4)>max_m4){
				max_m4=x.objective_values.get(4);
			}
		}
		double d0 = Math.pow((Math.min(max_m0, true_max[0])-Math.max(min_m0, true_min[0]))/(true_max[0]-true_min[0]), 2);
		double d1 = Math.pow((Math.min(max_m1, true_max[1])-Math.max(min_m1, true_min[1]))/(true_max[1]-true_min[1]), 2);
		double d2 = Math.pow((Math.min(max_m2, true_max[2])-Math.max(min_m2, true_min[2]))/(true_max[2]-true_min[2]), 2);
		double d3 = Math.pow((Math.min(max_m3, true_max[3])-Math.max(min_m3, true_min[3]))/(true_max[3]-true_min[3]), 2);
		double d4 = Math.pow((Math.min(max_m4, true_max[4])-Math.max(min_m4, true_min[4]))/(true_max[4]-true_min[4]), 2);
		double MS=Math.sqrt(0.2*(d0+d1+d2+d3+d4));
		return MS;
	}
	
	public double C(ArrayList<Individual> X1, ArrayList<Individual> X2){
		int covered_by_X1=0;
		for(Individual x2: X2){
			boolean covered=false;
			for(Individual x1: X1){
				if(x1.weakly_dominates(x2)){
					covered=true;
				}
			}
			if(covered){
				covered_by_X1+=1;
			}
		}
		double C=(double)covered_by_X1/X2.size();
		return C;
	}
	
	public void evaluate_population(ArrayList<Individual> X, double v, int evaluation_runs){
		for(Individual x:X){
			for(int d=0;d<x.fr.D;d++){
				for(int r=0;r<x.fr.R;r++){
					for(Surgery s:x.room_schedule[d][r]){
						s.duration.set_std(v);
						s.duration.set_sample_durations(evaluation_runs);
					}
				}
			}
			evaluate_individual(x, evaluation_runs);
		}
		double average_OR_overtime =0;
		double average_OR_idle_time = 0;
		double average_surgeon_overtime =0;
		double average_s_idle_time = 0;
		double average_waiting_cost = 0;
		
		double[] OR_overtime = new double[X.size()];
		double[] OR_idle_time = new double[X.size()];
		double[] surgeon_overtime = new double[X.size()];
		double[] s_idle_time = new double[X.size()];
		double[] waiting_cost = new double[X.size()];
		
		for(int i=0; i< X.size(); i++){
			OR_overtime[i] = X.get(i).mean.get(0);
			OR_idle_time[i] = X.get(i).mean.get(1);
			surgeon_overtime[i] = X.get(i).mean.get(2);
			s_idle_time[i] = X.get(i).mean.get(3);
			waiting_cost[i] = X.get(i).mean.get(4);
			
			average_OR_overtime += X.get(i).mean.get(0)/X.size();
			average_OR_idle_time += X.get(i).mean.get(1)/X.size();
			average_surgeon_overtime += X.get(i).mean.get(2)/X.size();
			average_s_idle_time += X.get(i).mean.get(3)/X.size();
			average_waiting_cost += X.get(i).mean.get(4)/X.size();
		}
		System.out.println();
		System.out.println(median(OR_overtime));
		System.out.println(first_quartile(OR_overtime));
		System.out.println(third_quartile(OR_overtime));
		System.out.println(low_whisker(OR_overtime));
		System.out.println(high_whisker(OR_overtime));
		System.out.println();
		
		System.out.println();
		System.out.println(median(OR_idle_time));
		System.out.println(first_quartile(OR_idle_time));
		System.out.println(third_quartile(OR_idle_time));
		System.out.println(low_whisker(OR_idle_time));
		System.out.println(high_whisker(OR_idle_time));
		System.out.println();
		
		System.out.println();
		System.out.println(median(surgeon_overtime));
		System.out.println(first_quartile(surgeon_overtime));
		System.out.println(third_quartile(surgeon_overtime));
		System.out.println(low_whisker(surgeon_overtime));
		System.out.println(high_whisker(surgeon_overtime));
		System.out.println();
		
		System.out.println();
		System.out.println(median(s_idle_time));
		System.out.println(first_quartile(s_idle_time));
		System.out.println(third_quartile(s_idle_time));
		System.out.println(low_whisker(s_idle_time));
		System.out.println(high_whisker(s_idle_time));
		System.out.println();
		
		System.out.println();
		System.out.println(median(waiting_cost));
		System.out.println(first_quartile(waiting_cost));
		System.out.println(third_quartile(waiting_cost));
		System.out.println(low_whisker(waiting_cost));
		System.out.println(high_whisker(waiting_cost));
		System.out.println();
		
		
		//System.out.println(median(OR_idle_time)+","+first_quartile(OR_idle_time)+","+third_quartile(OR_idle_time)+","+low_whisker(OR_idle_time)+","+high_whisker(OR_idle_time));
		//System.out.println(median(surgeon_overtime)+","+first_quartile(surgeon_overtime)+","+third_quartile(surgeon_overtime)+","+low_whisker(surgeon_overtime)+","+high_whisker(surgeon_overtime));
		//System.out.println(median(s_idle_time)+","+first_quartile(s_idle_time)+","+third_quartile(s_idle_time)+","+low_whisker(s_idle_time)+","+high_whisker(s_idle_time));
		//System.out.println(median(waiting_cost)+","+first_quartile(waiting_cost)+","+third_quartile(waiting_cost)+","+low_whisker(waiting_cost)+","+high_whisker(waiting_cost));
		//System.out.println("Next generation:");
		//System.out.println("\\boxplot{0}"+"{"+median(OR_overtime)+"}"+"{"+first_quartile(OR_overtime)+"}"+"{"+third_quartile(OR_overtime)+"}"+"{"+low_whisker(OR_overtime)+"}"+"{"+high_whisker(OR_overtime)+"}");
		//System.out.println("\\boxplot{0}"+"{"+median(OR_idle_time)+"}"+"{"+first_quartile(OR_idle_time)+"}"+"{"+third_quartile(OR_idle_time)+"}"+"{"+low_whisker(OR_idle_time)+"}"+"{"+high_whisker(OR_idle_time)+"}");
		//System.out.println("\\boxplot{0}"+"{"+median(surgeon_overtime)+"}"+"{"+first_quartile(surgeon_overtime)+"}"+"{"+third_quartile(surgeon_overtime)+"}"+"{"+low_whisker(surgeon_overtime)+"}"+"{"+high_whisker(surgeon_overtime)+"}");
		//System.out.println("\\boxplot{0}"+"{"+median(s_idle_time)+"}"+"{"+first_quartile(s_idle_time)+"}"+"{"+third_quartile(s_idle_time)+"}"+"{"+low_whisker(s_idle_time)+"}"+"{"+high_whisker(s_idle_time)+"}");
		//System.out.println("\\boxplot{0}"+"{"+median(waiting_cost)+"}"+"{"+first_quartile(waiting_cost)+"}"+"{"+third_quartile(waiting_cost)+"}"+"{"+low_whisker(waiting_cost)+"}"+"{"+high_whisker(waiting_cost)+"}");
		//System.out.println("Mean values: "+average_OR_overtime+", "+average_OR_idle_time+", "+average_surgeon_overtime+", "+average_s_idle_time+", "+average_waiting_cost);
		//System.out.println("Meadian values: "+median(OR_overtime)+", "+median(OR_idle_time)+", "+median(surgeon_overtime)+", "+median(s_idle_time)+", "+median(waiting_cost));
		//System.out.println("Low whisker values: "+low_whisker(OR_overtime)+", "+low_whisker(OR_idle_time)+", "+low_whisker(surgeon_overtime)+", "+low_whisker(s_idle_time)+", "+low_whisker(waiting_cost));
		//System.out.println("High whisker values: "+high_whisker(OR_overtime)+", "+high_whisker(OR_idle_time)+", "+high_whisker(surgeon_overtime)+", "+high_whisker(s_idle_time)+", "+high_whisker(waiting_cost));
	}
	
	public void evaluate_individual(Individual x, int evaluation_runs){
		//int runs = 100;
		Simulator sim = new Simulator(x.fr, x.room_schedule, x.surgeon_schedule, x.T);
		
		double average_OR_overtime =0;
		double average_OR_idle_time = 0;
		double average_surgeon_overtime =0;
		double average_s_idle_time = 0;
		double average_waiting_cost = 0;
		
		double[] OR_overtime = new double[evaluation_runs];
		double[] OR_idle_time = new double[evaluation_runs];
		double[] surgeon_overtime = new double[evaluation_runs];
		double[] s_idle_time = new double[evaluation_runs];
		double[] waiting_cost = new double[evaluation_runs];
		
		for(int run=0; run< evaluation_runs; run++){
			ArrayList<Double> simulated_fitness = sim.estimated_fitness(run);
			OR_overtime[run] = simulated_fitness.get(0);
			OR_idle_time[run] = simulated_fitness.get(1);
			surgeon_overtime[run] = simulated_fitness.get(2);
			s_idle_time[run] = simulated_fitness.get(3);
			waiting_cost[run] = simulated_fitness.get(4);
			
			average_OR_overtime += simulated_fitness.get(0)/evaluation_runs;
			average_OR_idle_time += simulated_fitness.get(1)/evaluation_runs;
			average_surgeon_overtime += simulated_fitness.get(2)/evaluation_runs;
			average_s_idle_time += simulated_fitness.get(3)/evaluation_runs;
			average_waiting_cost += simulated_fitness.get(4)/evaluation_runs;
		}
		
		x.set_objective_values(average_OR_overtime, average_OR_idle_time, average_surgeon_overtime, average_s_idle_time, average_waiting_cost);
		//x.print_objective_values();
		//System.out.println("Expected OR overtime: "+average_OR_overtime);
		//System.out.println("Expected OR idle time: "+average_OR_idle_time);
		//System.out.println("Expected surgeon overtime: "+average_surgeon_overtime);
		//System.out.println("Expected surgeon idle time: "+average_s_idle_time);
		//System.out.println("Expected waiting cost: "+average_waiting_cost);
		
		x.set_mean_objective_values(average_OR_overtime, average_OR_idle_time, average_surgeon_overtime, average_s_idle_time, average_waiting_cost);
		x.set_median_objective_values(median(OR_overtime), median(OR_idle_time), median(surgeon_overtime), median(s_idle_time), median(waiting_cost));
		x.set_low_whisker_objective_values(low_whisker(OR_overtime), low_whisker(OR_idle_time), low_whisker(surgeon_overtime), low_whisker(s_idle_time), low_whisker(waiting_cost));
		x.set_high_whisker_objective_values(high_whisker(OR_overtime), high_whisker(OR_idle_time), high_whisker(surgeon_overtime), high_whisker(s_idle_time), high_whisker(waiting_cost));
	}
	
	public double median(double[] simulation_results){
		Arrays.sort(simulation_results);
		return simulation_results[(int)simulation_results.length/2]; 
	}
	
	public double first_quartile(double[] simulation_results){
		Arrays.sort(simulation_results);
		return simulation_results[(int)(0.25*simulation_results.length)]; 
	}
	
	public double third_quartile(double[] simulation_results){
		Arrays.sort(simulation_results);
		return simulation_results[(int)(0.75*simulation_results.length)]; 
	}
	
	public double low_whisker(double[] simulation_results){
		Arrays.sort(simulation_results);
		return simulation_results[(int)(0.02*simulation_results.length)]; 
	}
	
	public double high_whisker(double[] simulation_results){
		Arrays.sort(simulation_results);
		return simulation_results[(int)(0.98*simulation_results.length)]; 
	}
	
	//HELPER METHODS
	public int SH(Individual x_i, Individual x_j, double share){
		double distance = x_i.equlidean_distance(x_j);
		if(distance <share){
			return 1;
		}
		else return 0;
	}
	
	public int niche_count(Individual x, ArrayList<Individual> X, double share){
		int nc=0;
		for(Individual x_j : X){
			nc+=SH(x, x_j, share);
		}
		return Math.max(nc-1, 0);
	}
	
	public double niche_count_std(ArrayList<Individual> X, double share){
		double average_niche_count=0;
		for(Individual x:X){
			average_niche_count+=(double)niche_count(x,X,share)/X.size();
		}
		double squared_diff_sum =0;
		for(Individual x:X){
			squared_diff_sum+=Math.pow(((double)niche_count(x,X,share)-average_niche_count),2);
		}
		return Math.sqrt(squared_diff_sum/(X.size()-1));
	}
	
	public int NT(double[] q, double mu,ArrayList<Individual> X){
		for(Individual x:X){
			if(inside_hyper_cube(q,mu,x)){
				return 1;
			}
		}
		return 0;
	}
	
	//Assumes M=5
	public void set_normalized_objective_values(ArrayList<Individual> X){
			double max_m_0 = 0;
			double max_m_1 = 0;
			double max_m_2 = 0;
			double max_m_3 = 0;
			double max_m_4 = 0;
			for(Individual x:X){
				if(x.objective_values.get(0)>max_m_0){
					max_m_0=x.objective_values.get(0);
				}
				if(x.objective_values.get(1)>max_m_1){
					max_m_1=x.objective_values.get(1);
				}
				if(x.objective_values.get(2)>max_m_2){
					max_m_2=x.objective_values.get(2);
				}
				if(x.objective_values.get(3)>max_m_3){
					max_m_3=x.objective_values.get(3);
				}
				if(x.objective_values.get(4)>max_m_4){
					max_m_4=x.objective_values.get(4);
				}
			}
			for(Individual x: X){
				if(x.normalized_objective_values.size()==0){
					x.normalized_objective_values.add(x.objective_values.get(0)/max_m_0);
					x.normalized_objective_values.add(x.objective_values.get(1)/max_m_1);
					x.normalized_objective_values.add(x.objective_values.get(2)/max_m_2);
					x.normalized_objective_values.add(x.objective_values.get(3)/max_m_3);
					x.normalized_objective_values.add(x.objective_values.get(4)/max_m_4);
				}
				else{
					x.normalized_objective_values.set(0,x.objective_values.get(0)/max_m_0);
					x.normalized_objective_values.set(1,x.objective_values.get(1)/max_m_1);
					x.normalized_objective_values.set(2,x.objective_values.get(2)/max_m_2);
					x.normalized_objective_values.set(3,x.objective_values.get(3)/max_m_3);
					x.normalized_objective_values.set(4,x.objective_values.get(4)/max_m_4);
				}
			}
		}
	
	//Assumes M=5
	public boolean inside_hyper_cube(double[] q, double mu, Individual x){
			boolean inside=false;
			if(x.normalized_objective_values.get(0)>=q[0] && x.normalized_objective_values.get(0)<=q[0]+mu){
				if(x.normalized_objective_values.get(1)>=q[1] && x.normalized_objective_values.get(1)<=q[1]+mu){
					if(x.normalized_objective_values.get(2)>=q[2] && x.normalized_objective_values.get(2)<=q[2]+mu){
						if(x.normalized_objective_values.get(3)>=q[3] && x.normalized_objective_values.get(3)<=q[3]+mu){
							if(x.normalized_objective_values.get(4)>=q[4] && x.normalized_objective_values.get(4)<=q[4]+mu){
								inside =true;
							}
						}
					}
				}
			}
			return inside;
		}
}
