package hSPEA2;

import java.util.ArrayList;
import java.util.Collections;

public class Simulator {
	
	FileReader fr;
	ArrayList<Surgery>[][] room_schedule;
	ArrayList<Surgery>[][] surgeon_schedule;
	double tau;
	
	public Simulator(FileReader f, ArrayList<Surgery>[][] r_schedule, ArrayList<Surgery>[][] k_schedule, double T){
		fr = f;
		room_schedule = r_schedule;
		surgeon_schedule = k_schedule;
		tau = T;
	}
	
	public void sort_by_starting_time(){
		for(int d=0; d<fr.D;d++){
			for(int r=0; r<fr.R;r++){
				if(room_schedule[d][r]!=null){
					Collections.sort(room_schedule[d][r], Surgery.COMPARE_BY_STARTING_TIME);
				}
			}
			for(int k=0; k<fr.K;k++){
				if(surgeon_schedule[d][k] != null){
					Collections.sort(surgeon_schedule[d][k], Surgery.COMPARE_BY_STARTING_TIME);
				}
			}
		}
	}
	
	
	public void estimated_schedule(int run){
		sort_by_starting_time();
		for(int d = 0; d<fr.D;d++){
			for(int r = 0; r<fr.R;r++){
				if(room_schedule[d][r].size()>0){
					room_schedule[d][r].get(0).set_realized_start_time(Double.max(room_schedule[d][r].get(0).scheduled_start_time-tau,0));
					room_schedule[d][r].get(0).set_realized_end_time(room_schedule[d][r].get(0).realized_start_time+room_schedule[d][r].get(0).duration.sample_duration.get(run));
				}

				for(int i = 1; i<room_schedule[d][r].size();i++){
					room_schedule[d][r].get(i).set_realized_start_time(Double.max(room_schedule[d][r].get(i-1).realized_end_time, room_schedule[d][r].get(i).scheduled_start_time-tau));
					room_schedule[d][r].get(i).set_realized_end_time(room_schedule[d][r].get(i).realized_start_time+room_schedule[d][r].get(i).duration.sample_duration.get(run));
				}
			}
			for(int k=0; k<fr.K;k++){
				for(int i = 1; i<surgeon_schedule[d][k].size();i++){
					surgeon_schedule[d][k].get(i).set_realized_start_time(Double.max(Double.max(surgeon_schedule[d][k].get(i-1).realized_end_time, surgeon_schedule[d][k].get(i).scheduled_start_time-tau),surgeon_schedule[d][k].get(i).realized_start_time));
					surgeon_schedule[d][k].get(i).set_realized_end_time(surgeon_schedule[d][k].get(i).realized_start_time+surgeon_schedule[d][k].get(i).duration.sample_duration.get(run));
				}
			}
			//New code
			int propagation =0;
			while(propagation <3){
			for(int r = 0; r<fr.R;r++){
				for(int i = 1; i<room_schedule[d][r].size();i++){
					room_schedule[d][r].get(i).set_realized_start_time(Double.max(room_schedule[d][r].get(i-1).realized_end_time, room_schedule[d][r].get(i).realized_start_time));
					room_schedule[d][r].get(i).set_realized_end_time(room_schedule[d][r].get(i).realized_start_time+room_schedule[d][r].get(i).duration.sample_duration.get(run));
				}
			}
			for(int k = 0; k<fr.K;k++){
				for(int i = 1; i<surgeon_schedule[d][k].size();i++){
					surgeon_schedule[d][k].get(i).set_realized_start_time(Double.max(surgeon_schedule[d][k].get(i-1).realized_end_time, surgeon_schedule[d][k].get(i).realized_start_time));
					surgeon_schedule[d][k].get(i).set_realized_end_time(surgeon_schedule[d][k].get(i).realized_start_time+surgeon_schedule[d][k].get(i).duration.sample_duration.get(run));
				}
			}
			propagation+=1;
			}
		}
	}	
	
	public ArrayList<Double> estimated_fitness(int run){
		estimated_schedule(run);
		ArrayList<Double> simulated_objective_values = new ArrayList<Double>();
		double OR_idle_time = 0;
		double s_idle_time = 0;
		double OR_overtime =0;
		double surgeon_overtime =0;
		double waiting_cost = fr.initial_waiting_cost;
		
		for(int d = 0; d<fr.D; d++){
			for(int r=0; r<fr.R; r++){
				double time_spent_on_surgery = 0;
				double latest_ending_time = 32;
				for(Surgery s : room_schedule[d][r]){
					waiting_cost -= waiting_cost_reduce(s.surgery_id, d);
					time_spent_on_surgery +=s.duration.sample_duration.get(run);
					latest_ending_time = Double.max(latest_ending_time, s.realized_end_time);
				}
				OR_overtime += Double.max(latest_ending_time - 32, 0);
				OR_idle_time += latest_ending_time - time_spent_on_surgery;
			}
			for(int k=0; k<fr.K; k++){
				double time_spent_on_surgery = 0;
				double latest_ending_time = 0;
				double earliest_starting_time = 32;
				if(surgeon_schedule[d][k].size()==0){
					earliest_starting_time = 0;
				}
				for(Surgery s : surgeon_schedule[d][k]){
					time_spent_on_surgery +=s.duration.sample_duration.get(run);
					latest_ending_time = Double.max(latest_ending_time, s.realized_end_time);
					earliest_starting_time = Double.min(earliest_starting_time, s.realized_start_time);
				}
				surgeon_overtime += Double.max(latest_ending_time - 32, 0);
				s_idle_time += latest_ending_time - earliest_starting_time - time_spent_on_surgery;
				if((latest_ending_time - earliest_starting_time - time_spent_on_surgery)<-0.01){
					System.out.println("Negative idle time: "+(latest_ending_time - earliest_starting_time - time_spent_on_surgery));
					System.out.println("Latest ending time: "+latest_ending_time);
					System.out.println("Earliest starting time: "+earliest_starting_time);
					System.out.println("Time spent on surgery: "+time_spent_on_surgery);
				}
			}
		}
		
		simulated_objective_values.add(OR_overtime);
		simulated_objective_values.add(OR_idle_time);
		simulated_objective_values.add(surgeon_overtime);
		simulated_objective_values.add(s_idle_time);
		simulated_objective_values.add(waiting_cost);
		return simulated_objective_values;
	}
	
	public double waiting_cost_reduce(int surgery, int d){
		return waiting_cost(fr.refferal_date[surgery],fr.deadline[surgery],10)-waiting_cost(fr.refferal_date[surgery],fr.deadline[surgery],d);
	}
	
	public double waiting_cost(double r, double g, double d){
		return Math.pow((d-(-r))/(g-(-r)), 2);
	}
}
